# == Synopsis
# jpegHeader.rb
#
# This will examine a valid JPEG file (specifically the file header)
# for embedded information. It prints data about the byte length of
# the header and the total file, image dimensions, and comments (if
# they exist). It will optionally dump the file in hex, using the
# --dump flag.
#
# TODO: Add support for EXIF data.
#
# == Usage
# ruby jpegHeader.rb [ -f <file> ] [ -d | --dump ] [ -h ] [ -v ]
# 
# == Author
# Beau Grantham
# beau@thedarkmere.net
# 
# == Version
# 0x01 2006/06/01

require 'rdoc/usage'
require 'optparse'

bytes = []
len   = 0
dump  = false
img   = ""

# RDoc
opts = OptionParser.new
opts.on("-h", "--help") { RDoc::usage }
opts.on("-v", "--version") { RDoc::usage('version') }
opts.on("-f", "--file FILE") { |str| img = str }
opts.on("-d", "--dump") { dump = true }
opts.parse(ARGV) rescue RDoc::usage('usage')
RDoc::usage('usage') if img.eql?("")

begin
  # Open the file and copy bytes (in hex) into an array
  File.open(img, "r") do |file|
    file.each_byte { |b| bytes.push('%02X' % b) }
  end
rescue SystemCallError
  # File not found (or can't be opened)
  puts "File not found!"
  puts "Check the file name and try again, or use the -h switch for help."
  exit
end

# A valid jpeg will start with FF D8
if !bytes[0].eql?("FF") and !bytes[1].eql?("D8")
  puts "Not a valid JPEG image."

  # Exit on an invalid filetype
  exit
end

# Find the end of the header, so we don't get false markers
# from the body of the image later on
for i in 0..bytes.length-1
  # Header ends at first FF C4 marker
  if bytes[i].eql?("FF") and bytes[i+1].eql?("C4")
    end_head = i
    break
  end
end

# If we should dump the file
if dump
  bytes.each { |b| print b + " " }
  2.times { puts }
end

# Print byte lengths
puts "Header length: " + end_head.to_s + " bytes"
puts "Total file length: " + bytes.length.to_s + " bytes"

# Scan the header for important markers
for i in 0..end_head
  # Dimensions marker:   FF C0  or  FF C2
  # Height is found at bytes 5 and 6 (highbyte and lowbyte) past the marker
  # Height is found at bytes 7 and 8 (highbyte and lowbyte) past the marker 
  if bytes[i].eql?("FF") and ( bytes[i+1].eql?("C0") or bytes[i+1].eql?("C2") )
    width  = (bytes[i+7].hex * 256) + bytes[i+8].hex
    height = (bytes[i+5].hex * 256) + bytes[i+6].hex
    
    # Print dimensions
    puts "Image dimensions: " + width.to_s + " x " + height.to_s + " px"
  end

  # Comments marker:   FF FE
  # Length is found at the two bytes (highbyte and lowbyte) afterwards
  # Comments are found immediatly after the length, up until length - 3
  if bytes[i].eql?("FF") and bytes[i+1].eql?("FE")
    len = (bytes[i+2].hex * 256) + bytes[i+3].hex
    
    # Print comments
    puts "Comments: "
    for j in 0..len-3
      print bytes[i+4+j].hex.chr
    end
    puts
  end
end

# No comments
puts "No Comments" if len == 0
